@@ -1,6 +1,6 @@
 /* aguix.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2020 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -45,6 +45,7 @@
 #include <list>
 #include <atomic>
 #include <memory>
+#include <functional>
 
 #include "aguixfont.hh"
 #include "aguixcolor.hh"
@@ -379,6 +380,11 @@ public:
                               size_t data_length );
     bool getStandardIconData( const unsigned long **return_data,
                               size_t *return_data_length ) const;
+
+    void closeBubbleHelp( AWindow *awin );
+
+    void pushPostMsgHandlerCB( std::function< void(void) > f );
+    void executeAfterMsgHandler( std::function< void(void) > f );
 private:
   Display *dsp;
   int scr;
@@ -649,6 +655,10 @@ private:
 
     const unsigned long *m_net_wm_icon_data = NULL;
     size_t m_net_wm_icon_data_length = 0;
+
+    bool msg_handler_active = false;
+
+    std::list< std::function< void(void) > > m_post_msg_handler_cbs;
 };
 
 #endif
