@@ -1,6 +1,6 @@
 /* awindow.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2020 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -334,16 +334,18 @@ bool AWindow::handleMessage(XEvent *E,Message *msg)
 #endif      
 	if ( ( getForbidInput() == true ) && ( conLock == false ) ) return returnvalue;
 	break;
-      case FocusOut:
+        case FocusOut:
 #ifdef USE_XIM
-	if ( inputcontext != NULL ) {
-	  if ( msg->window == win ) {
-	    XUnsetICFocus( inputcontext );
-	  }
-	}
-#endif      
-	if ( ( getForbidInput() == true ) && ( conLock == false ) ) return returnvalue;
-	break;
+            if ( inputcontext != NULL ) {
+                if ( msg->window == win ) {
+                    XUnsetICFocus( inputcontext );
+                }
+            }
+#endif
+            _aguix->closeBubbleHelp( this );
+
+            if ( ( getForbidInput() == true ) && ( conLock == false ) ) return returnvalue;
+            break;
     }
     
     if(msg->window==win) {
@@ -401,18 +403,19 @@ bool AWindow::handleMessage(XEvent *E,Message *msg)
        msg->type == KeyPress &&
        isTopLevel() == true &&
        callBackForbidden() == false ) {
-    std::list<key_listener_t>::iterator it;
     AGMessage *agmsg = AGUIX_allocAGMessage( msg );
-    
-    for ( it = key_listener.begin(); it != key_listener.end(); it++ ) {
-      //TODO pruefen, ob subwin ue1berhautp ein subwin von this is, also recursiv alle childs pruefen
-      if ( ( (*it).mode & KEYCB_WHENVISIBLE ) == 0 || (*it).subwin->isVisible() == true ) {
-        forbidCallBacks();
-        (*it).cb->run( (*it).subwin, *agmsg );
-        permitCallBacks();
-      }
-    }
-    AGUIX_freeAGMessage( agmsg );
+
+    _aguix->executeAfterMsgHandler( [ this, agmsg ] {
+        for ( auto &l : key_listener ) {
+            //TODO pruefen, ob subwin ue1berhautp ein subwin von this is, also recursiv alle childs pruefen
+            if ( ( l.mode & KEYCB_WHENVISIBLE ) == 0 || l.subwin->isVisible() == true ) {
+                forbidCallBacks();
+                l.cb->run( l.subwin, *agmsg );
+                permitCallBacks();
+            }
+        }
+        AGUIX_freeAGMessage( agmsg );
+    });
   }
 
   return returnvalue;
@@ -1164,6 +1167,11 @@ void AWindow::doDestroyStuff()
     }
     _aguix->Flush();
 
+    // to avoid a possible bug in xfw4
+    // (https://gitlab.xfce.org/xfce/xfwm4/-/issues/514) I call XSync
+    // here. AWindows are not opened/closed that often so it should be fine.
+    _aguix->xSync();
+
     if ( m_size_hints ) {
         XFree( m_size_hints );
         m_size_hints = NULL;
