@@ -1,6 +1,6 @@
 /* kartei.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2004-2005,2009 Ralf Hoffmann.
+ * Copyright (C) 2004-2019 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -74,6 +74,25 @@ void Kartei::setOption( AWindow *twin, unsigned int option, const char *optionna
   redraw();
 }
 
+void Kartei::updateOption( unsigned int option, const char *optionname )
+{
+    if ( optionname == NULL ) return;
+  
+    if ( option >= options.size() ) return;
+
+    optionkb->setOption( option, optionname );
+    optionkb->resize( optionkb->getMaxSize(), optionkb->getHeight() );
+    cont->readLimits();
+    if ( curOption == option ) showCurOption();
+    redraw();
+}
+
+void Kartei::updateOption( unsigned int option, const std::string &optionname )
+{
+    updateOption( option,
+                  optionname.c_str() );
+}
+
 void Kartei::showCurOption()
 {
   AWindow *twin, *otwin;
