@@ -1,6 +1,6 @@
 /* popupmenu.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2007-2015 Ralf Hoffmann.
+ * Copyright (C) 2007-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -930,10 +930,11 @@ bool PopUpMenu::isVisibleEntry( int pos ) const
     return false;
 }
 
-int PopUpMenu::checkForVisibleEntries( const std::string &filter )
+int PopUpMenu::checkForVisibleEntries( const std::string &filter, bool prefer_prefix )
 {
     std::string lower_filter = AGUIXUtils::tolower( filter );
 
+    int res_pos = -1;
     int pos = 0;
     for ( std::vector<InternPopUpEntry>::iterator it1 = m_entries.begin();
           it1 != m_entries.end();
@@ -945,8 +946,14 @@ int PopUpMenu::checkForVisibleEntries( const std::string &filter )
           case SUBMENU:
           case EDITABLE:
             lower_name = AGUIXUtils::tolower( it1->name );
-            if ( lower_name.find( lower_filter ) != std::string::npos ) {
-                return pos;
+            if ( prefer_prefix ) {
+                if ( AGUIXUtils::starts_with( lower_name, lower_filter ) ) {
+                    return pos;
+                }
+            }
+
+            if ( res_pos == -1 && lower_name.find( lower_filter ) != std::string::npos ) {
+                res_pos = pos;
             }
             break;
           default:
@@ -954,7 +961,7 @@ int PopUpMenu::checkForVisibleEntries( const std::string &filter )
         }
     }
     
-    return -1;
+    return res_pos;
 }
 
 void PopUpMenu::addStringToFilter( const char *str )
@@ -962,7 +969,9 @@ void PopUpMenu::addStringToFilter( const char *str )
     std::string temp_filter = m_infix_filter;
     temp_filter += str;
 
-    int res = checkForVisibleEntries( temp_filter );
+    bool prefer_prefix = m_infix_filter.empty();
+
+    int res = checkForVisibleEntries( temp_filter, prefer_prefix );
     if ( res >= 0 ) {
         m_infix_filter = temp_filter;
 
@@ -970,6 +979,8 @@ void PopUpMenu::addStringToFilter( const char *str )
             // set highlighted entry even if it is not visible as it
             // will jump to next visible automatically
             m_highlighted_entry = res;
+        } else if ( prefer_prefix ) {
+            setHighlightEntry( res );
         }
 
         updateInfixFilter();
