@@ -1,6 +1,6 @@
 /* util.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2017 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -27,6 +27,7 @@
 #include <functional>
 #include <stdlib.h>
 #include <cmath>
+#include "utf8.hh"
 
 int createLines( const char *text, char ***ret_lines, const char split )
 {
@@ -472,6 +473,29 @@ namespace AGUIXUtils
         }
     }
 
+    std::string remove_char( const std::string &str,
+                             char ch )
+    {
+        std::string res;
+        const char *in = str.c_str();
+
+        for ( ; *in != '\0'; ) {
+            size_t l = UTF8::getLenOfCharacter( in );
+
+            if ( l < 1 ) {
+                break;
+            } if ( l == 1 && *in == ch ) {
+                // skip
+                in++;
+            } else {
+                for ( size_t t = 0; t < l; t++ ) {
+                    res += *in++;
+                }
+            }
+        }
+        return res;
+    }
+
     std::string timeToString( time_t t )
     {
         char buf[128];
@@ -652,4 +676,31 @@ namespace AGUIXUtils
 
         return 0;
     }
+
+    std::string replace_percent_s_quoted( const std::string &src,
+                                          const std::string &replacement,
+                                          std::string *return_suffix )
+    {
+        // TODO it would be more accurate to check for backslashed %
+        // and so on to find the real, unprotected %s
+        size_t p = src.find( "%s" );
+
+        if ( p == std::string::npos ) {
+            return src;
+        }
+
+        std::string prefix( src, 0, p );
+        std::string suffix( src, p + 2 );
+
+        std::string res = AGUIX_catTrustedAndUnTrusted2( prefix,
+                                                         replacement );
+
+        if ( return_suffix ) {
+            *return_suffix = suffix;
+        } else {
+            res += suffix;
+        }
+
+        return res;
+    }
 }
