@@ -1,6 +1,6 @@
 /* configtokens.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2003-2020 Ralf Hoffmann.
+ * Copyright (C) 2003-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -452,7 +452,22 @@ struct mytoken tokenlist[]={
   { DIRECTORYPRESETS_WCP, "directorypresets" },
   { DIRECTORYPRESET_WCP, "directorypreset" },
   { HELPOP_WCP, "helpop" },
-  { PRIORITY_WCP, "priority" }
+  { PRIORITY_WCP, "priority" },
+  { AUTOFILTER_WCP, "autofilter" },
+  { ENABLE_CUSTOM_LVB_LINE_WCP, "enablecustomlvbline" },
+  { CUSTOM_LVB_LINE_WCP, "customlvbline" },
+  { CUSTOM_DIRECTORY_INFO_COMMAND_WCP, "customdirectoryinfocommand" },
+  { IMMEDIATE_FILTER_APPLY_WCP, "immediatefilterapply" },
+  { DISABLEBGCHECKPREFIX_WCP, "disablebgcheckprefix" },
+  { DISPLAYMODE_WCP, "displaymode" },
+  { SHOWSUBDIRS_WCP, "showsubdirs" },
+  { SHOWDIRECTSUBDIRS_WCP, "showdirectsubdirs" },
+  { INCLUDEALL_WCP, "includeall" },
+  { APPLYMODE_WCP, "applymode" },
+  { ADD_WCP, "add" },
+  { REMOVE_WCP, "remove" },
+  { HIDENONEXISTING_WCP, "hidenonexisting" },
+  { VIEWCOMMANDLOGOP_WCP, "viewcommandlogop" }
 };
 
 ArrayList *tokenlist_sorted = NULL;
