@@ -1,6 +1,6 @@
 /* copyop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2017 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -44,7 +44,6 @@ CopyOp::CopyOp() : FunctionProto()
   do_rename=false;
   same_dir=false;
   request_dest=false;
-  request_flags=false;
   overwrite=COPYOP_OVERWRITE_NORMAL;
   preserve_attr=true;
   vdir_preserve_dir_structure = false;
@@ -52,6 +51,7 @@ CopyOp::CopyOp() : FunctionProto()
   ensure_file_permissions = LEAVE_PERMISSIONS_UNMODIFIED;
   hasConfigure = true;
     m_category = FunctionProto::CAT_FILEOPS;
+    setRequestParameters( false );
 }
 
 CopyOp::~CopyOp()
@@ -67,12 +67,12 @@ CopyOp::duplicate() const
   ta->do_rename=do_rename;
   ta->same_dir=same_dir;
   ta->request_dest=request_dest;
-  ta->request_flags=request_flags;
   ta->overwrite=overwrite;
   ta->preserve_attr=preserve_attr;
   ta->vdir_preserve_dir_structure = vdir_preserve_dir_structure;
   ta->adjust_relative_symlinks = adjust_relative_symlinks;
   ta->ensure_file_permissions = ensure_file_permissions;
+  ta->setRequestParameters( requestParameters() );
   return ta;
 }
 
@@ -89,7 +89,7 @@ CopyOp::getName()
 }
 
 int
-CopyOp::run( WPUContext *wpu, ActionMessage *msg )
+CopyOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   ListerMode *lm1;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
@@ -121,7 +121,7 @@ CopyOp::save(Datei *fh)
   fh->configPutPairBool( "rename", do_rename );
   fh->configPutPairBool( "samedir", same_dir );
   fh->configPutPairBool( "requestdest", request_dest );
-  fh->configPutPairBool( "requestflags", request_flags );
+  fh->configPutPairBool( "requestflags", requestParameters() );
   switch(overwrite) {
     case COPYOP_OVERWRITE_ALWAYS:
       fh->configPutPair( "overwrite", "always" );
@@ -195,7 +195,7 @@ CopyOp::normalmodecopy( ActionMessage *am )
       }
   }
   
-  if(request_flags==true) {
+  if(requestParameters()==true) {
     if(doconfigure(1)!=0) cont=false;
   } else {
     // set values in t* variables
@@ -540,7 +540,7 @@ CopyOp::doconfigure(int mode)
                                                            catalog.getLocale( 1209 ), LABEL_RIGHT, 0 ), 0, 9, cincwnr );
 
   if(mode==0) {
-    rfcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( request_flags == true ) ? 1 : 0,
+    rfcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( requestParameters() == true ) ? 1 : 0,
 						      catalog.getLocale( 294 ), LABEL_RIGHT, 0 ), 0, 10, cincwnr );
   }
 
@@ -679,7 +679,7 @@ CopyOp::doconfigure(int mode)
       do_rename = rcb->getState();
       same_dir = sdcb->getState();
       request_dest = rdcb->getState();
-      request_flags = rfcb->getState();
+      setRequestParameters( rfcb->getState() );
       preserve_attr = pacb->getState();
       vdir_preserve_dir_structure = vdirpreserve_cb->getState();
 
@@ -804,11 +804,6 @@ void CopyOp::setRequestDest(bool nv)
   request_dest=nv;
 }
 
-void CopyOp::setRequestFlags(bool nv)
-{
-  request_flags=nv;
-}
-
 void CopyOp::setOverwrite(overwrite_t nv)
 {
   overwrite=nv;
@@ -841,5 +836,5 @@ bool CopyOp::isInteractiveRun() const
 
 void CopyOp::setInteractiveRun()
 {
-    request_flags = true;
+    setRequestParameters( true );
 }
