@@ -1187,7 +1187,7 @@ void Datei::configPutPairNum( const char *id, int num1, int num2 )
   putLine( str1.c_str() );
 }
 
-void Datei::configPutPairString( const char *id, const char *str )
+void Datei::configPutPairString( const char *id, const char *str, bool escape_newlines )
 {
   std::string str1;
   char *buf;
@@ -1206,6 +1206,11 @@ void Datei::configPutPairString( const char *id, const char *str )
     } else if ( str[i] == '\\' ) {
       buf[o++] = '\\';
       buf[o++] = '\\';
+    } else if ( str[i] == '\n' ) {
+        if ( escape_newlines ) {
+            buf[o++] = '\\';
+            buf[o++] = '\n';
+        }
     } else {
       buf[o++] = str[i];
     }
@@ -1249,7 +1254,7 @@ void Datei::configPutInfo( const char *str, bool semicolon )
   putLine( str1.c_str() );
 }
 
-void Datei::configPutString( const char *str, bool semicolon )
+void Datei::configPutString( const char *str, bool semicolon, bool escape_newlines )
 {
   std::string str1;
   char *buf;
@@ -1268,6 +1273,11 @@ void Datei::configPutString( const char *str, bool semicolon )
     } else if ( str[i] == '\\' ) {
       buf[o++] = '\\';
       buf[o++] = '\\';
+    } else if ( str[i] == '\n' ) {
+        if ( escape_newlines ) {
+            buf[o++] = '\\';
+            buf[o++] = '\n';
+        }
     } else {
       buf[o++] = str[i];
     }
