@@ -102,10 +102,10 @@ public:
   void configPutPair( const char *id, const char *info );
   void configPutPairNum( const char *id, int num );
   void configPutPairNum( const char *id, int num1, int num2 );
-  void configPutPairString( const char *id, const char *str );
+  void configPutPairString( const char *id, const char *str, bool escape_newlines = false);
   void configPutPairBool( const char *id, bool val );
   void configPutInfo( const char *str, bool semicolon );
-  void configPutString( const char *str, bool semicolon );
+  void configPutString( const char *str, bool semicolon, bool escape_newlines = false );
   void configOpenSection( const char *sectioname );
   void configCloseSection();
 
