@@ -1,6 +1,6 @@
 /* dircompareop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2017-2020 Ralf Hoffmann.
+ * Copyright (C) 2017-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -224,6 +224,10 @@ int DirCompareOp::compareEntry( NWC::FSEntry *my_entry,
 
     // regular file
 
+    if ( m_compare_mode == COMPARE_FILENAME ) {
+        return 0;
+    }
+
     if ( my_entry->stat_size() != other_entry->stat_size() ) {
         return 1;
     }
@@ -244,6 +248,17 @@ int DirCompareOp::key_compare( const compare_element &lhs,
                                const compare_element &rhs,
                                enum compare_key key )
 {
+    if ( ! lhs.entry->isReg() && rhs.entry->isReg() ) {
+        return -1;
+    } else if ( lhs.entry->isReg() && ! rhs.entry->isReg() ) {
+        return 1;
+    } else if ( ! lhs.entry->isReg() && ! rhs.entry->isReg() ) {
+        return StringComparator::compare( lhs.basename,
+                                          rhs.basename );
+    }
+
+    // both are regular files
+
     if ( key == FILESIZE ) {
         return rhs.entry->stat_size() - lhs.entry->stat_size();
     } else if ( key == FILESIZE_CHECKSUM ) {
@@ -264,10 +279,10 @@ int DirCompareOp::key_compare( const compare_element &lhs,
     }
 }
 
-void DirCompareOp::calc_checksum( compare_element &element ) const
+int DirCompareOp::calc_checksum( compare_element &element ) const
 {
     if ( ! element.entry->isReg() ) {
-        return;
+        return 1;
     }
 
 #ifdef HAVE_OPENSSL_SHA256
@@ -277,14 +292,14 @@ void DirCompareOp::calc_checksum( compare_element &element ) const
 
     if ( res != 1 ) {
         fprintf( stderr, "Failed to init checksum\n" );
-        return;
+        return 1;
     }
 
     Datei fh;
 
     if ( fh.open( element.entry->getFullname().c_str(),
                   "r" ) != 0 ) {
-        return;
+        return 1;
     }
 
     char buf[128 * 1024];
@@ -292,6 +307,8 @@ void DirCompareOp::calc_checksum( compare_element &element ) const
     size_t total_bytes_read = 0;
     size_t total_bytes = element.entry->stat_size();
 
+    bool stopped = false;
+
     for (;;) {
         size_t bufsize = sizeof( buf );
 
@@ -330,6 +347,7 @@ void DirCompareOp::calc_checksum( compare_element &element ) const
         }
 
         if ( m_progress_win->check() ) {
+            stopped = true;
             break;
         }
     }
@@ -338,7 +356,12 @@ void DirCompareOp::calc_checksum( compare_element &element ) const
     if ( res != 1 ) {
         fprintf( stderr, "Failed to get checksum\n" );
     }
+
+    if ( stopped ) {
+        return -1;
+    }
 #endif
+    return 0;
 }
 
 int DirCompareOp::compareDirs( std::shared_ptr< NWC::Dir > my_dir,
@@ -353,10 +376,12 @@ int DirCompareOp::compareDirs( std::shared_ptr< NWC::Dir > my_dir,
     int unequal_count = 0;
 
     if ( my_equal ) {
+        my_equal->clear();
         my_equal->resize( my_dir->size() );
     }
 
     if ( other_equal ) {
+        other_equal->clear();
         other_equal->resize( other_dir->size() );
     }
 
@@ -433,10 +458,20 @@ int DirCompareOp::compareDirs( std::shared_ptr< NWC::Dir > my_dir,
 
     if ( m_compare_mode == COMPARE_CHECKSUM ) {
         for ( auto &e : my_dir_elements ) {
-            calc_checksum( e );
+            if ( calc_checksum( e ) < 0 ) {
+                m_stop = true;
+                break;
+            }
+            m_progress_win->finishedEntry();
         }
-        for ( auto &e : other_dir_elements ) {
-            calc_checksum( e );
+        if ( ! m_stop ) { 
+            for ( auto &e : other_dir_elements ) {
+                if ( calc_checksum( e ) < 0 ) {
+                    m_stop = true;
+                    break;
+                }
+                m_progress_win->finishedEntry();
+            }
         }
     }
 
@@ -465,7 +500,9 @@ int DirCompareOp::compareDirs( std::shared_ptr< NWC::Dir > my_dir,
             other_pos++;
             unequal_count++;
 
-            m_progress_win->finishedEntry();
+            if ( m_compare_mode != COMPARE_CHECKSUM ) {
+                m_progress_win->finishedEntry();
+            }
         };
 
         if ( other_pos >= other_dir->size() ) {
@@ -476,7 +513,10 @@ int DirCompareOp::compareDirs( std::shared_ptr< NWC::Dir > my_dir,
             unequal_count++;
 
             my_pos++;
-            m_progress_win->finishedEntry();
+            
+            if ( m_compare_mode != COMPARE_CHECKSUM ) {
+                m_progress_win->finishedEntry();
+            }
         } else {
             if ( key_compare( my_dir_elements[my_pos],
                               other_dir_elements[other_pos], cmp_key ) == 0 ) {
@@ -496,7 +536,11 @@ int DirCompareOp::compareDirs( std::shared_ptr< NWC::Dir > my_dir,
                                        other_dir_elements[other_end_pos], cmp_key ) == 0; other_end_pos++ ) {
                 }
 
+                std::vector< bool > other_matched;
+                other_matched.resize( other_end_pos - other_start_pos );
+
                 for ( size_t my_block_pos = my_start_pos; my_block_pos < my_end_pos; my_block_pos++ ) {
+                    bool my_matched = false;
                     for ( size_t other_block_pos = other_start_pos; other_block_pos < other_end_pos; other_block_pos++ ) {
                         bool cmp_res = false;
 
@@ -507,33 +551,45 @@ int DirCompareOp::compareDirs( std::shared_ptr< NWC::Dir > my_dir,
                                            other_res_vdir,
                                            cmp_key ) == 0 ) {
                             cmp_res = true;
-                        }
-
-                        if ( my_equal ) {
-                            my_equal->at( my_dir_elements[my_block_pos].orig_pos ) = cmp_res;
-                        }
-                        if ( other_equal && cmp_res == true) {
-                            other_equal->at( other_dir_elements[other_block_pos].orig_pos ) = cmp_res;
+                        } else {
+                            unequal_count++;
                         }
 
                         if ( cmp_res ) {
-                            if ( my_res_vdir ) {
+                            if ( my_equal ) {
+                                my_equal->at( my_dir_elements[my_block_pos].orig_pos ) = cmp_res;
+                            }
+                            if ( other_equal ) {
+                                other_equal->at( other_dir_elements[other_block_pos].orig_pos ) = cmp_res;
+                            }
+
+                            if ( my_res_vdir && ! my_matched ) {
                                 my_res_vdir->add( *my_dir_elements[my_block_pos].entry );
                             }
+
+                            other_matched.at( other_block_pos - other_start_pos ) = true;
+
+                            my_matched = true;
                         }
                     }
-                    m_progress_win->finishedEntry();
+
+                    if ( m_compare_mode != COMPARE_CHECKSUM ) {
+                        m_progress_win->finishedEntry();
+                    }
                 }
 
                 // if any of the other elements of same key are
                 // "equal", put them in the virtual dir
                 for ( size_t other_block_pos = other_start_pos; other_block_pos < other_end_pos; other_block_pos++ ) {
-                    if ( other_equal->at( other_dir_elements[other_block_pos].orig_pos ) == false ) {
+                    if ( ! other_matched.at( other_block_pos - other_start_pos )  ) {
                         if ( other_res_vdir ) {
                             other_res_vdir->add( *other_dir_elements[other_block_pos].entry );
                         }
+
+                        if ( m_compare_mode != COMPARE_CHECKSUM ) {
+                            m_progress_win->finishedEntry();
+                        }
                     }
-                    m_progress_win->finishedEntry();
                 }
 
                 my_pos = my_end_pos;
@@ -546,7 +602,10 @@ int DirCompareOp::compareDirs( std::shared_ptr< NWC::Dir > my_dir,
                 unequal_count++;
 
                 my_pos++;
-                m_progress_win->finishedEntry();
+
+                if ( m_compare_mode != COMPARE_CHECKSUM ) {
+                    m_progress_win->finishedEntry();
+                }
             }
         }
 
@@ -564,7 +623,9 @@ int DirCompareOp::compareDirs( std::shared_ptr< NWC::Dir > my_dir,
         }
         unequal_count++;
 
-        m_progress_win->finishedEntry();
+        if ( m_compare_mode != COMPARE_CHECKSUM ) {
+            m_progress_win->finishedEntry();
+        }
 
         if ( m_progress_win->check() ) {
             m_stop = true;
@@ -575,7 +636,7 @@ int DirCompareOp::compareDirs( std::shared_ptr< NWC::Dir > my_dir,
 }
 
 int
-DirCompareOp::run( WPUContext *wpu, ActionMessage *msg )
+DirCompareOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     if ( msg->mode == msg->AM_MODE_DNDACTION ) {
         return 0;
