@@ -1,6 +1,6 @@
 /* dmcacheentrynwc.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2012-2019 Ralf Hoffmann.
+ * Copyright (C) 2012-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -769,3 +769,42 @@ int DMCacheEntryNWC::changeBasenameOfEntry( const std::string &basename,
                                          m_entries[pos].getEntryID(),
                                          true );
 }
+
+const std::string &DMCacheEntryNWC::getCustomDirectoryInfo()
+{
+    if ( m_custom_directory_info_future.valid() ) {
+        if ( ! customDirectoryInfoPending() ) {
+            m_custom_directory_info = m_custom_directory_info_future.get();
+            m_custom_directory_info_state = CDI_DONE;
+        }
+    }
+
+    return m_custom_directory_info;
+}
+
+void DMCacheEntryNWC::setCustomDirectoryInfoFuture( std::future< std::string > f )
+{
+    m_custom_directory_info_future = std::move( f );
+    m_custom_directory_info_state = CDI_CALCULATING;
+}
+
+bool DMCacheEntryNWC::customDirectoryInfoPending()
+{
+    if ( ! m_custom_directory_info_future.valid() ) {
+        return false;
+    }
+
+    std::future_status status = m_custom_directory_info_future.wait_for( std::chrono::duration<int>::zero() );
+
+    if ( status == std::future_status::ready ) {
+        m_custom_directory_info_state = CDI_RESULT_AVAILABLE;
+        return false;
+    } else {
+        return true;
+    }
+}
+
+DMCacheEntryNWC::custom_directory_info_state DMCacheEntryNWC::customDirectoryInfoState() const
+{
+    return m_custom_directory_info_state;
+}
