@@ -1,6 +1,6 @@
 /* enterdirop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2009-2014 Ralf Hoffmann.
+ * Copyright (C) 2009-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -77,7 +77,7 @@ EnterDirOp *EnterDirOp::duplicate() const
     return ta;
 }
 
-int EnterDirOp::run( WPUContext *wpu, ActionMessage *msg )
+int EnterDirOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     Lister *l1;
     ListerMode *lm1;
@@ -310,6 +310,15 @@ int EnterDirOp::configure()
                 case AG_STRINGGADGET_DEACTIVATE:
                     rcyb->setOption( 2 );
                     break;
+                case AG_KEYPRESSED:
+                    if ( win->isParent( msg->key.window, false ) == true ) {
+                        switch ( msg->key.key ) {
+                            case XK_Escape:
+                                endmode = 1;
+                                break;
+                        }
+                    }
+                    break;
             }
             aguix->ReplyMessage( msg );
         }
