@@ -1,6 +1,6 @@
 /* listermode.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2017 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -148,8 +148,8 @@ void ListerMode::registerCommand( const std::string &command,
     m_commands[ command ] = callback;
 
     if ( ! description.empty() &&
-         m_command_info.count( command ) == 0 ) {
-        m_command_info[ command ] = std::make_tuple( description, category );
+         m_command_info.count( std::make_pair( command, false ) ) == 0 ) {
+        m_command_info[ std::make_pair( command, false ) ] = std::make_tuple( description, category );
     }
 }
 
@@ -176,8 +176,8 @@ void ListerMode::registerCommand( const std::string &command,
     m_commands_with_args[command] = callback;
 
     if ( ! description.empty() &&
-         m_command_info.count( command ) == 0 ) {
-        m_command_info[ command ] = std::make_tuple( description, category );
+         m_command_info.count( std::make_pair( command, true ) ) == 0 ) {
+        m_command_info[ std::make_pair( command, true ) ] = std::make_tuple( description, category );
     }
 }
 
@@ -206,8 +206,17 @@ void ListerMode::runCommandWithStrArgs( const std::string &command_str )
         }
 
         if ( ! command.empty() ) {
-            if ( string_args.size() > 0 &&
-                 m_commands_with_args.count( command ) > 0 ) {
+            if ( m_commands.count( command ) == 0 &&
+                 m_commands_with_args.count( command ) > 0 &&
+                 string_args.empty() ) {
+                std::string text = AGUIXUtils::formatStringToString( catalog.getLocale( 1390 ),
+                                                                     command.c_str() );
+
+                parentlister->getWorker()->getRequester()->request( catalog.getLocale( 123 ),
+                                                                    text.c_str(),
+                                                                    catalog.getLocale( 11 ) );
+            } else if ( string_args.size() > 0 &&
+                        m_commands_with_args.count( command ) > 0 ) {
                 runCommand( command, string_args );
             } else {
                 runCommand( command );
@@ -258,10 +267,11 @@ std::list< WorkerTypes::reg_command_info_t > ListerMode::getListOfCommandsWithou
     for ( auto &e : m_commands ) {
         WorkerTypes::reg_command_info_t t = { .name = e.first,
                                               .description = "",
-                                              .category = FunctionProto::CAT_OTHER };
-        if ( m_command_info.count( e.first ) > 0 ) {
-            t.description = std::get<0>( m_command_info.at( e.first ) );
-            t.category = std::get<1>( m_command_info.at( e.first ) );
+                                              .category = FunctionProto::CAT_OTHER,
+                                              .has_args = false };
+        if ( m_command_info.count( std::make_pair( e.first, false ) ) > 0 ) {
+            t.description = std::get<0>( m_command_info.at( std::make_pair( e.first, false ) ) );
+            t.category = std::get<1>( m_command_info.at( std::make_pair( e.first, false ) ) );
         }
 
         res.push_back( t );
@@ -270,15 +280,36 @@ std::list< WorkerTypes::reg_command_info_t > ListerMode::getListOfCommandsWithou
     return res;
 }
 
-WorkerTypes::reg_command_info_t ListerMode::getCommandInfo( const std::string &command ) const
+std::list< WorkerTypes::reg_command_info_t > ListerMode::getListOfCommandsWithArgs() const
 {
-    if ( m_command_info.count( command ) == 0 ) return {};
+    std::list< WorkerTypes::reg_command_info_t > res;
+
+    for ( auto &e : m_commands_with_args ) {
+        WorkerTypes::reg_command_info_t t = { .name = e.first,
+                                              .description = "",
+                                              .category = FunctionProto::CAT_OTHER,
+                                              .has_args = true };
+        if ( m_command_info.count( std::make_pair( e.first, true ) ) > 0 ) {
+            t.description = std::get<0>( m_command_info.at( std::make_pair( e.first, true ) ) );
+            t.category = std::get<1>( m_command_info.at( std::make_pair( e.first, true ) ) );
+        }
+
+        res.push_back( t );
+    }
+
+    return res;
+}
+
+WorkerTypes::reg_command_info_t ListerMode::getCommandInfo( const std::string &command, bool args ) const
+{
+    if ( m_command_info.count( std::make_pair( command, args ) ) == 0 ) return {};
 
     WorkerTypes::reg_command_info_t res;
 
     res.name = command;
-    res.description = std::get<0>( m_command_info.at( command ) );
-    res.category = std::get<1>( m_command_info.at( command ) );
+    res.description = std::get<0>( m_command_info.at( std::make_pair( command, args ) ) );
+    res.has_args = args;
+    res.category = std::get<1>( m_command_info.at( std::make_pair( command, args ) ) );
 
     return res;
 }
