@@ -1,6 +1,6 @@
 /* modifytabsop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2009-2018 Ralf Hoffmann.
+ * Copyright (C) 2009-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -65,7 +65,7 @@ ModifyTabsOp::getName()
 }
 
 int
-ModifyTabsOp::run( WPUContext *wpu, ActionMessage *msg )
+ModifyTabsOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     ListerMode *lm1;
     if ( msg->mode != msg->AM_MODE_DNDACTION ) {
@@ -252,6 +252,15 @@ ModifyTabsOp::configure()
                     if ( msg->button.button == okb ) endmode = 0;
                     else if ( msg->button.button == cb ) endmode = 1;
                     break;
+                case AG_KEYPRESSED:
+                    if ( win->isParent( msg->key.window, false ) == true ) {
+                        switch ( msg->key.key ) {
+                            case XK_Escape:
+                                endmode = 1;
+                                break;
+                        }
+                    }
+                    break;
             }
             aguix->ReplyMessage( msg );
         }
