@@ -1,6 +1,6 @@
 /* normalops.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2017 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -36,6 +36,7 @@
 #include "fileentry.hh"
 #include "stringmatcher_fnmatch.hh"
 #include "pers_kvp.hh"
+#include "aguix/choosebutton.h"
 
 /******
   ChangeListerSetOp
@@ -71,7 +72,7 @@ ChangeListerSetOp *ChangeListerSetOp::duplicate() const
   return ta;
 }
 
-int ChangeListerSetOp::run( WPUContext *wpu, ActionMessage *msg )
+int ChangeListerSetOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   Lister *l1;
   switch(mode) {
@@ -180,6 +181,15 @@ int ChangeListerSetOp::configure()
           if(msg->button.button==okb) endmode=0;
           else if(msg->button.button==cb) endmode=1;
           break;
+        case AG_KEYPRESSED:
+            if ( win->isParent( msg->key.window, false ) == true ) {
+                switch ( msg->key.key ) {
+                    case XK_Escape:
+                        endmode = 1;
+                        break;
+                }
+            }
+            break;
       }
       aguix->ReplyMessage(msg);
     }
@@ -280,7 +290,7 @@ SwitchListerOp *SwitchListerOp::duplicate() const
   return ta;
 }
 
-int SwitchListerOp::run( WPUContext *wpu, ActionMessage *msg )
+int SwitchListerOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   Lister *l1;
   l1 = msg->getWorker()->getOtherLister( msg->getWorker()->getActiveLister() );
@@ -320,7 +330,7 @@ const char *FilterSelectOp::getName()
 
 FilterSelectOp::FilterSelectOp() : FunctionProto()
 {
-    filter = "*";
+    m_filter = "*";
     current_filter = "";
   
     if ( instance_counter == 0 ) {
@@ -330,7 +340,7 @@ FilterSelectOp::FilterSelectOp() : FunctionProto()
     hasConfigure = true;
     m_category = FunctionProto::CAT_SELECTIONS;
 
-    m_mode = 0;
+    m_mode = FILTER_SELECT;
 }
 
 FilterSelectOp::~FilterSelectOp()
@@ -344,40 +354,52 @@ FilterSelectOp::~FilterSelectOp()
 FilterSelectOp *FilterSelectOp::duplicate() const
 {
   FilterSelectOp *ta=new FilterSelectOp();
-  ta->filter = filter;
+  ta->m_filter = m_filter;
+  ta->m_auto_filter = m_auto_filter;
+  ta->setImmediateFilterApply( immediateFilterApply() );
   return ta;
 }
 
-int FilterSelectOp::run( WPUContext *wpu, ActionMessage *msg )
+int FilterSelectOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
-  Lister *l1;
-  ListerMode *lm1;
-  if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    l1 = msg->getWorker()->getActiveLister();
-    if(l1!=NULL) {
-      lm1=l1->getActiveMode();
-      if(lm1!=NULL) {
-        if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
-            std::list< NM_specialsourceExt > extfilelist;
-            std::string entry_basename;
+    Lister *l1;
+    ListerMode *lm1;
 
-            lm1->getSelFiles( extfilelist, ListerMode::LM_GETFILES_ONLYACTIVE );
-            for ( auto &ss1 : extfilelist ) {
-                if ( ss1.entry() ) {
-                    entry_basename = ss1.entry()->name;
-                }
-            }
-
-            if ( requestfilter( entry_basename ) == true ) {
-                vdm->selectByFilter( current_filter, true );
-            }
-        } else {
-            lm1->not_supported();
-        }
-      }
+    if ( msg->mode == msg->AM_MODE_DNDACTION ) {
+        return 0;
     }
-  }
-  return 0;
+
+    l1 = msg->getWorker()->getActiveLister();
+    if ( ! l1 ) {
+        return 0;
+    }
+
+    lm1=l1->getActiveMode();
+    if ( ! lm1 ) {
+        return 0;
+    }
+
+    if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+        std::list< NM_specialsourceExt > extfilelist;
+        std::string entry_basename;
+
+        lm1->getSelFiles( extfilelist, ListerMode::LM_GETFILES_ONLYACTIVE );
+        for ( auto &ss1 : extfilelist ) {
+            if ( ss1.entry() ) {
+                entry_basename = ss1.entry()->name;
+            }
+        }
+
+        if ( m_immediate_filter_apply ) {
+            vdm->selectByFilter( m_filter, m_mode == FILTER_SELECT ? true : false );
+        } else if ( requestfilter( entry_basename ) == true ) {
+            vdm->selectByFilter( current_filter, m_mode == FILTER_SELECT ? true : false );
+        }
+    } else {
+        lm1->not_supported();
+    }
+
+    return 0;
 }
 
 const char *FilterSelectOp::getDescription()
@@ -405,21 +427,29 @@ int FilterSelectOp::configure()
   win->create();
   _freesafe(tstr);
   
-  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 2 ), true );
+  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 4 ), true );
   ac1->setMinSpace( 5 );
   ac1->setMaxSpace( 5 );
 
-  AContainer *ac1_1 = ac1->add( new AContainer( win, 2, 1 ), 0, 0 );
+  auto autofilter_cb = ac1->addWidget( new ChooseButton( aguix, 0, 0, m_auto_filter,
+                                                         catalog.getLocale( 1370 ), LABEL_RIGHT, 0 ),
+                                       0, 0, AContainer::CO_INCWNR );
+
+  AContainer *ac1_1 = ac1->add( new AContainer( win, 2, 1 ), 0, 1 );
   ac1_1->setMinSpace( 5 );
   ac1_1->setMaxSpace( 5 );
   ac1_1->setBorderWidth( 0 );
 
   ac1_1->add( new Text( aguix, 0, 0, catalog.getLocale( 349 ) ), 0, 0, cfix );
   sg = (StringGadget*)ac1_1->add( new StringGadget( aguix, 0, 0, 100,
-						    filter.c_str(),
-						    0 ), 1, 0, cincw );
+                                                    m_filter.c_str(),
+                                                    0 ), 1, 0, cincw );
   
-  AContainer *ac1_2 = ac1->add( new AContainer( win, 2, 1 ), 0, 1 );
+  auto immediate_cb = ac1->addWidget( new ChooseButton( aguix, 0, 0, m_immediate_filter_apply,
+                                                        catalog.getLocale( 1384 ), LABEL_RIGHT, 0 ),
+                                      0, 2, AContainer::CO_INCWNR );
+
+  AContainer *ac1_2 = ac1->add( new AContainer( win, 2, 1 ), 0, 3 );
   ac1_2->setMinSpace( 5 );
   ac1_2->setMaxSpace( -1 );
   ac1_2->setBorderWidth( 0 );
@@ -440,6 +470,13 @@ int FilterSelectOp::configure()
   AGMessage *msg;
   while((msg=aguix->GetMessage(NULL))!=NULL) aguix->ReplyMessage(msg);
   int ende=0;
+
+  if ( autofilter_cb->getState() == true ) {
+      ac1_1->hide();
+  } else {
+      ac1_1->show();
+  }
+
   while(ende==0) {
     msg=aguix->WaitMessage(win);
     if(msg!=NULL) {
@@ -448,12 +485,37 @@ int FilterSelectOp::configure()
       } else if(msg->type==AG_BUTTONCLICKED) {
         if(msg->button.button==okb) ende=1;
         else if(msg->button.button==cancelb) ende=-1;
+      } else if ( msg->type == AG_CHOOSECLICKED ) {
+          if ( msg->choose.button == autofilter_cb ) {
+              if ( autofilter_cb->getState() == true ) {
+                  ac1_1->hide();
+                  immediate_cb->setState( false );
+              } else {
+                  ac1_1->show();
+              }
+          } else if ( msg->choose.button == immediate_cb ) {
+              if ( immediate_cb->getState() == true ) {
+                  autofilter_cb->setState( false );
+                  ac1_1->show();
+              }
+          }
+      } else if ( msg->type == AG_KEYPRESSED ) {
+          if ( win->isParent( msg->key.window, false ) == true ) {
+              switch ( msg->key.key ) {
+                  case XK_Escape:
+                      ende = -1;
+                      break;
+              }
+          }
+          break;
       }
       aguix->ReplyMessage(msg);
     }
   }
   if(ende==1) {
-      filter = sg->getText();
+      m_filter = sg->getText();
+      m_auto_filter = autofilter_cb->getState();
+      m_immediate_filter_apply = immediate_cb->getState();
   }
   delete win;
   return (ende==1)?0:1;
@@ -462,11 +524,19 @@ int FilterSelectOp::configure()
 bool FilterSelectOp::save(Datei *fh)
 {
   if ( fh == NULL ) return false;
-  fh->configPutPairString( "filter", filter.c_str() );
+  fh->configPutPairString( "filter", m_filter.c_str() );
+  if ( m_auto_filter == false ) {
+      fh->configPutPairBool( "autofilter", m_auto_filter );
+  }
+
+  if ( immediateFilterApply() == true ) {
+      fh->configPutPairBool( "immediatefilterapply", immediateFilterApply() );
+  }
+
   return true;
 }
 
-void FilterSelectOp::setMode( int nm )
+void FilterSelectOp::setMode( filter_select_mode nm )
 {
     m_mode = nm;
 }
@@ -494,7 +564,7 @@ bool FilterSelectOp::requestfilter( const std::string &fullname )
                    AContainer::ACONT_MINW;
 
   win = new AWindow( aguix, 10, 10, 10, 10,
-                     catalog.getLocale( ( m_mode == 1 ) ? 269 : 268 ),
+                     catalog.getLocale( ( m_mode == FILTER_DESELECT ) ? 269 : 268 ),
                      AWindow::AWINDOW_DIALOG );
   win->create();
 
@@ -519,7 +589,12 @@ bool FilterSelectOp::requestfilter( const std::string &fullname )
   lv->setDisplayFocus( true );
   lv->setAcceptFocus( true );
 
-  std::string default_value = current_filter.empty() ? filter : current_filter;
+  std::string default_value = m_filter;
+
+  if ( m_auto_filter && ! current_filter.empty() ) {
+      default_value = current_filter;
+  }
+
   bool matched = false;
 
   std::list< std::string > cur_history = m_history->getList();
@@ -530,7 +605,7 @@ bool FilterSelectOp::requestfilter( const std::string &fullname )
       lv->setText( row, 0, *it1 );
       lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
 
-      if ( ! matched && ! fullname.empty() && *it1 != "*" ) {
+      if ( m_auto_filter && ! matched && ! fullname.empty() && *it1 != "*" ) {
           StringMatcherFNMatch matcher;
 
           matcher.setMatchString( *it1 );
@@ -547,7 +622,7 @@ bool FilterSelectOp::requestfilter( const std::string &fullname )
   lv->showRow( cur_history.size() - 1 );
   lv->redraw();
 
-  if ( ! matched ) {
+  if ( m_auto_filter && ! matched ) {
       std::string::size_type dot_pos = fullname.rfind( "." );
 
       if ( dot_pos != std::string::npos ) {
@@ -718,7 +793,22 @@ void FilterSelectOp::addHistoryItem( const std::string &str )
 void FilterSelectOp::setFilter( const char *str )
 {
   if ( str == NULL ) return;
-  filter = str;
+  m_filter = str;
+}
+
+void FilterSelectOp::setAutoFilter( const bool nv )
+{
+    m_auto_filter = nv;
+}
+
+void FilterSelectOp::setImmediateFilterApply( bool nv )
+{
+    m_immediate_filter_apply = nv;
+}
+
+bool FilterSelectOp::immediateFilterApply() const
+{
+    return m_immediate_filter_apply;
 }
 
 /*************
@@ -744,7 +834,7 @@ const char *FilterUnSelectOp::getName()
 FilterUnSelectOp::FilterUnSelectOp() : FilterSelectOp()
 {
     m_category = FunctionProto::CAT_SELECTIONS;
-    setMode( 1 );
+    setMode( FILTER_DESELECT );
 }
 
 FilterUnSelectOp::~FilterUnSelectOp()
@@ -754,122 +844,17 @@ FilterUnSelectOp::~FilterUnSelectOp()
 FilterUnSelectOp *FilterUnSelectOp::duplicate() const
 {
   FilterUnSelectOp *ta=new FilterUnSelectOp();
-  ta->filter = filter;
+  ta->m_filter = m_filter;
+  ta->m_auto_filter = m_auto_filter;
+  ta->setImmediateFilterApply( immediateFilterApply() );
   return ta;
 }
 
-int FilterUnSelectOp::run( WPUContext *wpu, ActionMessage *msg )
-{
-  Lister *l1;
-  ListerMode *lm1;
-  if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    l1 = msg->getWorker()->getActiveLister();
-    if(l1!=NULL) {
-      lm1=l1->getActiveMode();
-      if(lm1!=NULL) {
-        if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
-            std::list< NM_specialsourceExt > extfilelist;
-            std::string entry_basename;
-
-            lm1->getSelFiles( extfilelist, ListerMode::LM_GETFILES_ONLYACTIVE );
-            for ( auto &ss1 : extfilelist ) {
-                if ( ss1.entry() ) {
-                    entry_basename = ss1.entry()->name;
-                }
-            }
-
-            if ( requestfilter( entry_basename ) == true ) {
-                vdm->selectByFilter( current_filter, false );
-            }
-        } else {
-            lm1->not_supported();
-        }
-      }
-    }
-  }
-  return 0;
-}
-
 const char *FilterUnSelectOp::getDescription()
 {
   return catalog.getLocale(1263);
 }
 
-int FilterUnSelectOp::configure()
-{
-  StringGadget *sg;
-  AGUIX *aguix = Worker::getAGUIX();
-  AWindow *win;
-  char *tstr;
-  const int cincw = AContainer::ACONT_MINH +
-                    AContainer::ACONT_MINW +
-                    AContainer::ACONT_MAXH;
-  const int cfix = AContainer::ACONT_MINH +
-                   AContainer::ACONT_MINW +
-                   AContainer::ACONT_MAXH +
-                   AContainer::ACONT_MAXW;
-
-  tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(getDescription())+1);
-  sprintf(tstr,catalog.getLocale(293),getDescription());
-  win = new AWindow( aguix, 10, 10, 10, 10, tstr, AWindow::AWINDOW_DIALOG );
-  win->create();
-  _freesafe(tstr);
-  
-  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 2 ), true );
-  ac1->setMinSpace( 5 );
-  ac1->setMaxSpace( 5 );
-
-  AContainer *ac1_1 = ac1->add( new AContainer( win, 2, 1 ), 0, 0 );
-  ac1_1->setMinSpace( 5 );
-  ac1_1->setMaxSpace( 5 );
-  ac1_1->setBorderWidth( 0 );
-
-  ac1_1->add( new Text( aguix, 0, 0, catalog.getLocale( 349 ) ), 0, 0, cfix );
-
-  sg = (StringGadget*)ac1_1->add( new StringGadget( aguix, 0, 0, 100,
-						    filter.c_str(),
-						    0 ), 1, 0, cincw );
-  
-  AContainer *ac1_2 = ac1->add( new AContainer( win, 2, 1 ), 0, 1 );
-  ac1_2->setMinSpace( 5 );
-  ac1_2->setMaxSpace( -1 );
-  ac1_2->setBorderWidth( 0 );
-  Button *okb =(Button*)ac1_2->add( new Button( aguix,
-                                                0,
-                                                0,
-                                                catalog.getLocale( 11 ),
-                                                0 ), 0, 0, cfix );
-  Button *cancelb = (Button*)ac1_2->add( new Button( aguix,
-						     0,
-						     0,
-						     catalog.getLocale( 8 ),
-						     0 ), 1, 0, cfix );
-
-  win->setDoTabCycling( true );
-  win->contMaximize( true );
-  win->show();
-  AGMessage *msg;
-  while((msg=aguix->GetMessage(NULL))!=NULL) aguix->ReplyMessage(msg);
-  int ende=0;
-  while(ende==0) {
-    msg=aguix->WaitMessage(win);
-    if(msg!=NULL) {
-      if(msg->type==AG_CLOSEWINDOW) {
-        if(msg->closewindow.window==win->getWindow()) ende=-1;
-      } else if(msg->type==AG_BUTTONCLICKED) {
-        if(msg->button.button==okb) ende=1;
-        else if(msg->button.button==cancelb) ende=-1;
-      }
-      aguix->ReplyMessage(msg);
-    }
-  }
-  if(ende==1) {
-      filter = sg->getText();
-  }
-  delete win;
-  return (ende==1)?0:1;
-}
-
 /*************
  Ende FilterUnSelectOp
 **************/
@@ -904,7 +889,7 @@ Path2OSideOp *Path2OSideOp::duplicate() const
   return ta;
 }
 
-int Path2OSideOp::run( WPUContext *wpu, ActionMessage *msg )
+int Path2OSideOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     Lister *l1;
     ListerMode *lm1;
@@ -981,7 +966,7 @@ QuitOp *QuitOp::duplicate() const
   return ta;
 }
 
-int QuitOp::run( WPUContext *wpu, ActionMessage *msg )
+int QuitOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   msg->getWorker()->quit( mode == Q_QUICK_QUIT ? 1 : 0 );
   return 0;
@@ -1068,6 +1053,15 @@ int QuitOp::configure()
           if(msg->button.button==okb) endmode=0;
           else if(msg->button.button==cb) endmode=1;
           break;
+        case AG_KEYPRESSED:
+            if ( win->isParent( msg->key.window, false ) == true ) {
+                switch ( msg->key.key ) {
+                    case XK_Escape:
+                        endmode = 1;
+                        break;
+                }
+            }
+            break;
       }
       aguix->ReplyMessage(msg);
     }
