@@ -152,13 +152,9 @@ namespace NWC
 
   std::string FSEntry::getDirname() const
   {
-    const std::string &fn = getFullname();
-  
-    std::string::size_type pos = fn.rfind( "/" );
-    if ( pos == std::string::npos )
-      return std::string( "." );
+      const std::string &fn = getFullname();
 
-    return std::string( fn, 0, pos );
+      return NWC::Path::dirname( fn );
   }
 
   std::string FSEntry::getExtension() const
