@@ -269,8 +269,8 @@ int NWCEntrySelectionState::sortfunction( const NWCEntrySelectionState &e1,
               }
           }
 
-          NWCFlagProducer fpr1( &e1, NULL );
-          NWCFlagProducer fpr2( &e2, NULL );
+          auto fpr1 = std::make_shared< NWCFlagProducer >( &e1, nullptr );
+          auto fpr2 = std::make_shared< NWCFlagProducer >( &e2, nullptr );
           FlagReplacer rpl1( fpr1 );
           FlagReplacer rpl2( fpr2 );
 
@@ -488,6 +488,7 @@ std::string NWCEntrySelectionState::getStringRepresentation( WorkerTypes::listco
     bool sfds = false;
     int ost, x;
     char nbuf[ A_BYTESFORNUMBER( loff_t ) ];
+    struct tm temptm;
 
     if ( ! fse ) return res;
   
@@ -591,15 +592,16 @@ std::string NWCEntrySelectionState::getStringRepresentation( WorkerTypes::listco
             fse->getDestination( res );
             break;
         case WorkerTypes::LISTCOL_MOD:
-            timeptr = localtime( &( timestamp = fse->stat_lastmod() ) );
+            timeptr = localtime_r( &( timestamp = fse->stat_lastmod() ), &temptm );
             wconfig->writeDateToString( res, timeptr );
+            
             break;
         case WorkerTypes::LISTCOL_ACC:
-            timeptr = localtime( &( timestamp = fse->stat_lastaccess() ) );
+            timeptr = localtime_r( &( timestamp = fse->stat_lastaccess() ), &temptm );
             wconfig->writeDateToString( res, timeptr );
             break;
         case WorkerTypes::LISTCOL_CHANGE:
-            timeptr = localtime( &( timestamp = fse->stat_lastchange() ) );
+            timeptr = localtime_r( &( timestamp = fse->stat_lastchange() ), &temptm );
             wconfig->writeDateToString( res, timeptr );
             break;
         case WorkerTypes::LISTCOL_INODE:
