@@ -1,6 +1,6 @@
 /* ownop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2014 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -23,6 +23,7 @@
 #include "worker.h"
 #include "filereq.h"
 #include "datei.h"
+#include "aguix/lowlevelfunc.h"
 #include "aguix/button.h"
 #include "aguix/fieldlistview.h"
 #include "aguix/stringgadget.h"
@@ -98,7 +99,7 @@ OwnOp::getName()
 }
 
 int
-OwnOp::run( WPUContext *wpu, ActionMessage *msg )
+OwnOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   externprog( wpu, msg );
 
@@ -275,7 +276,7 @@ OwnOp::configure()
   fb = (Button*)ac1_2->add( new Button( aguix, 0, 0, "O", 0 ), 4, 0, cfix );
   fb->setBubbleHelpText( catalog.getLocale( 1132 ) );
   
-  AContainer *ac1_3 = ac1->add( new AContainer( win, 4, 1 ), 0, 9 );
+  AContainer *ac1_3 = ac1->add( new AContainer( win, 5, 1 ), 0, 9 );
   ac1_3->setMinSpace( 0 );
   ac1_3->setMaxSpace( 0 );
   ac1_3->setBorderWidth( 0 );
@@ -285,6 +286,7 @@ OwnOp::configure()
   ac1_3->setMaxWidth( 5, 1, 0 );
   comreqb2 = (Button*)ac1_3->add( new Button( aguix, 0, 0, catalog.getLocale( 458 ), 0 ), 2, 0, cfix );
   sg[1] = (StringGadget*)ac1_3->add( new StringGadget( aguix, 0, 0, 100, view_str, 0 ), 3, 0, cincw );
+  auto view_help_b = ac1_3->addWidget( new Button( aguix, 0, 0, "?", 0 ), 4, 0, cfix );
   
   if ( cyb->getSelectedOption() == 3 ) {
       sg[1]->show();
@@ -396,6 +398,10 @@ OwnOp::configure()
                 _freesafe( newstr2 );
               }
             }
+          } else if ( msg->button.button == view_help_b ) {
+              req->request( catalog.getLocale( 125 ),
+                            catalog.getLocale( 1399 ),
+                            catalog.getLocale( 11 ) );
           }
           break;
 	case AG_STRINGGADGET_DEACTIVATE:
@@ -431,6 +437,15 @@ OwnOp::configure()
                 }
             }
             break;
+          case AG_KEYPRESSED:
+              if ( win->isParent( msg->key.window, false ) == true ) {
+                  switch ( msg->key.key ) {
+                      case XK_Escape:
+                          endmode = 1;
+                          break;
+                  }
+              }
+              break;
       }
       aguix->ReplyMessage(msg);
     }
@@ -632,7 +647,7 @@ void OwnOp::setFlagsAreRelativeToBaseDir( bool nv )
     m_flags_are_relative_to_base_dir = nv;
 }
 
-void OwnOp::externprog( WPUContext *wpu, ActionMessage *msg )
+void OwnOp::externprog( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     char *tmpname, *tmpoutput, *tstr;
     Datei *fp;
@@ -665,8 +680,6 @@ void OwnOp::externprog( WPUContext *wpu, ActionMessage *msg )
 
         RefCount< GenericCallbackArg< void, int > > pea( NULL );
       
-        wpu->ref();
-
         if ( ownstart == OWNOP_SHOW_OUTPUT_INT ) {
             tmpoutput_lnk = Datei::createTMPHardlink( tmpoutput );
             pea = new ProcessExitAction( true,
@@ -693,19 +706,21 @@ void OwnOp::externprog( WPUContext *wpu, ActionMessage *msg )
             lastsize[0] = wpu->filelistRealSize( false );
             lastsize[1] = wpu->filelistRealSize( true );
 
+            std::string quoted_tmpoutput = AGUIX_catTrustedAndUnTrusted2( "", tmpoutput );
+
             for ( int loopnr = 1;; loopnr++ ) {
                 std::string res_str;
-                if ( wpu->parse( com_str, res_str, EXE_STRING_LEN - strlen( tmpoutput ) - 3,
+                if ( wpu->parse( com_str, res_str, EXE_STRING_LEN - quoted_tmpoutput.size() - 4,
                                  true, WPUContext::PERSIST_FLAGS ) == WPUContext::PARSE_SUCCESS ) {
                     string1 = Worker_secureCommandForShell( res_str.c_str() );
                     if ( string1.length() > 0 ) {
                         if ( ownstart == OWNOP_SHOW_OUTPUT ||
                              ownstart == OWNOP_SHOW_OUTPUT_INT ) {
-                            tstr = (char*)_allocsafe( strlen( string1.c_str() ) + 3 + strlen( tmpoutput ) + 1 );
-                            if ( loopnr == 1 ) sprintf( tstr, "%s >%s", string1.c_str(), tmpoutput );
-                            else sprintf( tstr, "%s >>%s", string1.c_str(), tmpoutput );
-                            fp->putString( tstr);
-                            _freesafe( tstr );
+                            std::string tstr2 = string1;
+                            if ( loopnr == 1 ) tstr2 += " >";
+                            else tstr2 += " >>";
+                            tstr2 += quoted_tmpoutput;
+                            fp->putString( tstr2.c_str() );
                         } else {
                             fp->putString( string1.c_str() );
                         }
@@ -742,8 +757,7 @@ void OwnOp::externprog( WPUContext *wpu, ActionMessage *msg )
                     }
                     exestr = Worker::getTerminalCommand( tmpname, tmpoutput, remove_after_exec );
                 } else {
-                    exestr = "/bin/sh ";
-                    exestr += tmpname;
+                    exestr = AGUIXUtils::replace_percent_s_quoted( "/bin/sh %s", tmpname, NULL );
                 }
                 if ( ownstart == OWNOP_SHOW_OUTPUT ) {
                     useint = true;
@@ -752,8 +766,15 @@ void OwnOp::externprog( WPUContext *wpu, ActionMessage *msg )
                             if ( strstr( view_str, "%s" ) != NULL ) useint = false;
                         }
                     }
-                    if ( useint == true ) fp->putStringExt( OUTPUT_BIN, tmpoutput );
-                    else fp->putStringExt( view_str, tmpoutput );
+
+                    std::string real_view_str;
+
+                    if ( useint == true ) {
+                        real_view_str = AGUIXUtils::replace_percent_s_quoted( OUTPUT_BIN, tmpoutput, NULL );
+                    } else {
+                        real_view_str = AGUIXUtils::replace_percent_s_quoted( view_str, tmpoutput, NULL );
+                    }
+                    fp->putString( real_view_str.c_str() );
                     fp->putString( "\n" );
                 }
                 fp->close();
