@@ -1,6 +1,6 @@
 /* processexitaction.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2010-2011 Ralf Hoffmann.
+ * Copyright (C) 2010-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -42,23 +42,16 @@ private:
 ProcessExitAction::ProcessExitAction( bool show_file,
                                       const std::string &tmp_name,
                                       Worker *worker,
-                                      WPUContext *wpu ) : m_show_file( show_file ),
-                                                          m_tmp_name( tmp_name ),
-                                                          m_worker( worker ),
-                                                          m_wpu( wpu )
+                                      std::shared_ptr< WPUContext > wpu )
+    : m_show_file( show_file ),
+      m_tmp_name( tmp_name ),
+      m_worker( worker ),
+      m_wpu( wpu )
 {
 }
 
 ProcessExitAction::~ProcessExitAction()
 {
-    /* in case callback is not called */
-    if ( m_wpu != NULL ) {
-        if ( m_wpu->unref() < 1 ) {
-            delete m_wpu;
-        }
-
-        m_wpu = NULL;
-    }
 }
 
 void ProcessExitAction::callback( int status )
@@ -79,11 +72,5 @@ void ProcessExitAction::callback( int status )
         fv.view( l, cb );
     }
 
-    if ( m_wpu != NULL ) {
-        if ( m_wpu->unref() < 1 ) {
-            delete m_wpu;
-        }
-
-        m_wpu = NULL;
-    }
+    m_wpu = nullptr;
 }
