@@ -1,6 +1,6 @@
 /* searchop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006-2014 Ralf Hoffmann.
+ * Copyright (C) 2006-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -78,7 +78,7 @@ SearchOp::getName()
 }
 
 int
-SearchOp::run( WPUContext *wpu, ActionMessage *msg )
+SearchOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     prepareWindow( msg );
 
@@ -236,6 +236,15 @@ int SearchOp::configure()
                         sg->setText( DEFAULT_EDIT_COMMAND );
                     }
                     break;
+                case AG_KEYPRESSED:
+                    if ( win->isParent( msg->key.window, false ) == true ) {
+                        switch ( msg->key.key ) {
+                            case XK_Escape:
+                                endmode = 1;
+                                break;
+                        }
+                    }
+                    break;
             }
             aguix->ReplyMessage( msg );
         }
