@@ -739,19 +739,19 @@ void SearchOpBG::editResult()
 
 void SearchOpBG::editFile( const std::string &filename, int line_number )
 {
-    FlagReplacer::FlagProducerMap fl;
+    auto fl = std::make_shared< FlagReplacer::FlagProducerMap >();
     NWC::FSEntry fse( filename );
     char linestr[ A_BYTESFORNUMBER( int ) ];
 
     sprintf( linestr, "%d", line_number + 1 );
 
-    fl.registerFlag( "f", fse.getBasename() );
-    fl.registerFlag( "F", filename );
-    fl.registerFlag( "l", linestr );
+    fl->registerFlag( "f", fse.getBasename() );
+    fl->registerFlag( "F", filename );
+    fl->registerFlag( "l", linestr );
 
     std::string scriptdir = Worker::getDataDir();
     scriptdir += "/scripts";
-    fl.registerFlag( "scripts", scriptdir );
+    fl->registerFlag( "scripts", scriptdir );
   
     FlagReplacer rpl( fl );
   
@@ -759,7 +759,13 @@ void SearchOpBG::editFile( const std::string &filename, int line_number )
 
     std::unique_ptr<ExeClass> ec( new ExeClass() );
 
-    ec->cdDir( fse.getDirname().c_str() );
+    auto dirname = fse.getDirname();
+    if ( dirname.empty() ) {
+        ec->cdDir( "." );
+    } else {
+        ec->cdDir( dirname.c_str() );
+    }
+
     ec->addCommand( "%s", exestr.c_str() );
     ec->execute();
 }
