@@ -1,6 +1,6 @@
 /* setfilterop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2017 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -75,7 +75,6 @@ private:
 SetFilterOp::SetFilterOp() : FunctionProto()
 {
   filtermode=INCLUDE_FILTER;
-  request_flags=false;
   filter = "";
 
   if ( instance_counter == 0 ) {
@@ -103,6 +102,8 @@ SetFilterOp::SetFilterOp() : FunctionProto()
   m_worker = NULL;
 
     m_category = FunctionProto::CAT_FILELIST;
+
+    setRequestParameters( false );
 }
 
 SetFilterOp::~SetFilterOp()
@@ -120,7 +121,6 @@ SetFilterOp::duplicate() const
 {
   SetFilterOp *ta = new SetFilterOp();
   ta->filtermode=filtermode;
-  ta->request_flags=request_flags;
   ta->filter = filter;
 
   ta->m_bookmark_label = m_bookmark_label;
@@ -129,6 +129,7 @@ SetFilterOp::duplicate() const
   ta->m_change_filters = m_change_filters;
   ta->m_change_bookmarks = m_change_bookmarks;
   ta->m_query_label = m_query_label;
+  ta->setRequestParameters( requestParameters() );
   return ta;
 }
 
@@ -145,7 +146,7 @@ SetFilterOp::getName()
 }
 
 int
-SetFilterOp::run( WPUContext *wpu, ActionMessage *msg )
+SetFilterOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   ListerMode *lm1;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
@@ -170,7 +171,7 @@ bool
 SetFilterOp::save(Datei *fh)
 {
   if ( fh == NULL ) return false;
-  fh->configPutPairBool( "requestflags", request_flags );
+  fh->configPutPairBool( "requestflags", requestParameters() );
   switch(filtermode) {
     case EXCLUDE_FILTER:
       fh->configPutPair( "mode", "exclude" );
@@ -239,7 +240,7 @@ SetFilterOp::normalmodesf( ActionMessage *am )
         return 1;
     }
     
-    if ( request_flags == true ) {
+    if ( requestParameters() == true ) {
         if ( doconfigure( 1 ) != 0 ) cont = false;
     } else {
         // set values in t* variables
@@ -526,7 +527,7 @@ SetFilterOp::doconfigure(int mode)
                                                                                catalog.getLocale( 850 ), LABEL_RIGHT, 0 ), 0, 3, AContainer::CO_INCWNR );
 
   if(mode==0) {
-    rfcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20,( request_flags == true ) ? 1 : 0,
+    rfcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20,( requestParameters() == true ) ? 1 : 0,
 						      catalog.getLocale( 294 ), LABEL_RIGHT, 0 ), 0, 4, cincwnr );
   }
 
@@ -684,7 +685,7 @@ SetFilterOp::doconfigure(int mode)
       m_t_query_label = query_label_cb->getState();
     } else {
       // store in normal variables
-      request_flags = rfcb->getState();
+      setRequestParameters( rfcb->getState() );
       filter = sg->getText();
       switch(fmcyb->getSelectedOption()) {
         case 1:
@@ -782,11 +783,6 @@ void SetFilterOp::addHistoryItem( const std::string &str )
     }
 }
 
-void SetFilterOp::setRequestFlags( bool nv )
-{
-  request_flags = nv;
-}
-
 void SetFilterOp::setFilter( const char *nv )
 {
   if ( nv == NULL ) return;
@@ -1019,5 +1015,5 @@ bool SetFilterOp::isInteractiveRun() const
 
 void SetFilterOp::setInteractiveRun()
 {
-    request_flags = true;
+    setRequestParameters( true );
 }
