@@ -1,6 +1,6 @@
 /* dirsizeop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2014 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -61,7 +61,7 @@ ShowDirCacheOp::getName()
 }
 
 int
-ShowDirCacheOp::run( WPUContext *wpu, ActionMessage *msg )
+ShowDirCacheOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   ListerMode *lm1;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
