@@ -1,6 +1,6 @@
 /* startprogop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2020 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -99,7 +99,7 @@ StartProgOp::getName()
 }
 
 int
-StartProgOp::run( WPUContext *wpu, ActionMessage *msg )
+StartProgOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
     Lister *l1 = msg->getWorker()->getActiveLister();
@@ -174,7 +174,7 @@ static std::string protectForParsing( const char *str1 )
 }
 
 int
-StartProgOp::startprog( WPUContext *wpu, ActionMessage *am )
+StartProgOp::startprog( std::shared_ptr< WPUContext > wpu, ActionMessage *am )
 {
   ListerMode *lm1 = NULL;
 
@@ -249,14 +249,10 @@ StartProgOp::startprog( WPUContext *wpu, ActionMessage *am )
       if ( m1->_start == STARTPROGOP_SHOW_OUTPUT_INT ) {
           tmpoutput_lnk = Datei::createTMPHardlink( tmpoutput );
 
-          wpu->ref();
-
           pea = new ProcessExitAction( true,
                                        tmpoutput_lnk,
                                        am->getWorker(), wpu );
       } else {
-          wpu->ref();
-
           pea = new ProcessExitAction( false,
                                        "",
                                        am->getWorker(), wpu );
@@ -277,9 +273,11 @@ StartProgOp::startprog( WPUContext *wpu, ActionMessage *am )
         if ( m1->_global == false ) fp.putString( "./" );
 
         std::string res_str;
+        std::string quoted_tmpoutput = AGUIX_catTrustedAndUnTrusted2( "", tmpoutput );
+
         if ( wpu->parse( m1->_command.c_str(),
                          res_str,
-                         EXE_STRING_LEN - strlen( tmpoutput ) - 3,
+                         EXE_STRING_LEN - quoted_tmpoutput.size() - 3,
                          true,
                          WPUContext::PERSIST_FLAGS ) == WPUContext::PARSE_SUCCESS ) {
             string1 = Worker_secureCommandForShell( res_str.c_str() );
@@ -303,7 +301,7 @@ StartProgOp::startprog( WPUContext *wpu, ActionMessage *am )
           if ( m1->_start == STARTPROGOP_SHOW_OUTPUT ||
                m1->_start == STARTPROGOP_SHOW_OUTPUT_INT ) {
             string1 += " >";
-            string1 += tmpoutput;
+            string1 += quoted_tmpoutput;
           }
           fp.putString( string1.c_str() );
           fp.putString( "\n" );
@@ -322,15 +320,25 @@ StartProgOp::startprog( WPUContext *wpu, ActionMessage *am )
 
           exestr = Worker::getTerminalCommand( tmpname, tmpoutput, remove_after_exec );
         } else {
-            exestr = "/bin/sh ";
-            exestr += tmpname;
+            exestr = AGUIXUtils::replace_percent_s_quoted( "/bin/sh %s", tmpname, NULL );
         }
         if ( m1->_start == STARTPROGOP_SHOW_OUTPUT ) {
           useint = true;
-          if ( m1->_view_str.length() > 2 )
-            if ( strstr( m1->_view_str.c_str(), "%s" ) != NULL ) useint = false;
-          if ( useint == true ) fp.putStringExt( OUTPUT_BIN, tmpoutput );
-          else fp.putStringExt( m1->_view_str.c_str(), tmpoutput );
+          if ( m1->_view_str.length() > 2 ) {
+              if ( strstr( m1->_view_str.c_str(), "%s" ) != NULL ) {
+                  useint = false;
+              }
+          }
+
+          std::string real_view_str;
+
+          if ( useint == true ) {
+              real_view_str = AGUIXUtils::replace_percent_s_quoted( OUTPUT_BIN, tmpoutput, NULL );
+          } else {
+              real_view_str = AGUIXUtils::replace_percent_s_quoted( m1->_view_str, tmpoutput, NULL );
+          }
+          fp.putString( real_view_str.c_str() );
+
           fp.putString( "\n" );
         }
         fp.close();
