@@ -20,8 +20,8 @@
  */
 
 #include "textviewmode.hh"
-#include <aguix/acontainer.h>
-#include <aguix/textview.h>
+#include "aguix/acontainer.h"
+#include "aguix/textview.h"
 #include "worker_locale.h"
 #include "wconfig.h"
 #include "worker.h"
