@@ -1,6 +1,6 @@
 /* volumemanagerop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2009,2011,2013 Ralf Hoffmann.
+ * Copyright (C) 2009-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -59,7 +59,7 @@ const char *VolumeManagerOp::getName()
     return name;
 }
 
-int VolumeManagerOp::run( WPUContext *wpu, ActionMessage *msg )
+int VolumeManagerOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     Lister *l1;
     ListerMode *lm1;
