@@ -20,8 +20,8 @@
  */
 
 #include "wcfiletype.hh"
-#include <aguix/aguix.h>
-#include <aguix/util.h>
+#include "aguix/aguix.h"
+#include "aguix/util.h"
 #include "datei.h"
 #include "functionproto.h"
 #include "simplelist.hh"
@@ -38,11 +38,6 @@ WCFiletype::WCFiletype()
   usefiledesc=false;
   for(i=1;i<64;i++) filedesc[i]=-1;
   filedesc[0]=-2;
-  dndaction=new List();
-  doubleclickaction=new List();
-  showaction=new List();
-  rawshowaction=new List();
-  for(i=0;i<10;i++) useraction[i]=new List();
   internID = NORMALTYPE;
 
   patternIgnoreCase = false;
@@ -87,32 +82,12 @@ void WCFiletype::setinternID(int newid)
 
 WCFiletype::~WCFiletype()
 {
-  int i;
   if(name!=NULL) _freesafe(name);
 
 #ifdef HAVE_REGEX
   freeCompPatternReg();
 #endif
   if(pattern!=NULL) _freesafe(pattern);
-  freecoms(dndaction);
-  freecoms(doubleclickaction);
-  freecoms(showaction);
-  freecoms(rawshowaction);
-  for(i=0;i<10;i++) freecoms(useraction[i]);
-  delete dndaction;
-  delete doubleclickaction;
-  delete showaction;
-  delete rawshowaction;
-  for(i=0;i<10;i++) delete useraction[i];
-
-  std::map<std::string, List *>::iterator it1;
-
-  for ( it1 = customaction.begin();
-        it1 != customaction.end();
-        it1++ ) {
-      freecoms( it1->second );
-      delete it1->second;
-  }
 
   if ( extCond != NULL ) _freesafe( extCond );
 
@@ -143,12 +118,8 @@ WCFiletype *WCFiletype::duplicate() const
   tft->setRawShowActions(rawshowaction);
   for(i=0;i<10;i++) tft->setUserActions(i,useraction[i]);
 
-  std::map<std::string, List *>::const_iterator it2;
-
-  for ( it2 = customaction.begin();
-        it2 != customaction.end();
-        it2++ ) {
-      tft->setCustomActions( it2->first, it2->second );
+  for ( auto &key : customaction ) {
+      tft->setCustomActions( key.first, key.second );
   }
 
   tft->setinternID(internID);
@@ -179,6 +150,8 @@ WCFiletype *WCFiletype::duplicate() const
 
   tft->setPatternIsCommaSeparated( m_pattern_is_comma_separated );
 
+  tft->setPriority( m_priority );
+
   return tft;
 }
 
@@ -254,127 +227,72 @@ short *WCFiletype::getFiledesc()
   return filedesc;
 }
 
-List *WCFiletype::getDNDActions()
+command_list_t WCFiletype::getDNDActions()
 {
   return dndaction;
 }
 
-List *WCFiletype::getDoubleClickActions()
+command_list_t WCFiletype::getDoubleClickActions()
 {
   return doubleclickaction;
 }
 
-List *WCFiletype::getShowActions()
+command_list_t WCFiletype::getShowActions()
 {
   return showaction;
 }
 
-List *WCFiletype::getRawShowActions()
+command_list_t WCFiletype::getRawShowActions()
 {
   return rawshowaction;
 }
 
-List *WCFiletype::getUserActions(int id)
+command_list_t WCFiletype::getUserActions(int id)
 {
-  if((id<0)||(id>9)) return NULL;
+    if((id<0)||(id>9)) return {};
   return useraction[id];
 }
 
-void WCFiletype::setDNDActions(List *ndnda)
+void WCFiletype::setDNDActions( const command_list_t &ndnda )
 {
-  freecoms(dndaction);
-  FunctionProto *fp;
-  int id=ndnda->initEnum();
-  fp=(FunctionProto*)ndnda->getFirstElement(id);
-  while(fp!=NULL) {
-    dndaction->addElement(fp->duplicate());
-    fp=(FunctionProto*)ndnda->getNextElement(id);
-  }
-  ndnda->closeEnum(id);
+    dndaction.clear();
+    for ( auto &fp : ndnda ) {
+        dndaction.push_back( std::shared_ptr< FunctionProto >( fp->duplicate() ) );
+    }
 }
 
-void WCFiletype::addComDNDAction( const FunctionProto *fp )
+void WCFiletype::setDoubleClickActions( const command_list_t &ndca )
 {
-  if ( fp == NULL ) return;
-  dndaction->addElement( fp->duplicate() );
+    doubleclickaction.clear();
+    for ( auto &fp : ndca ) {
+        doubleclickaction.push_back( std::shared_ptr< FunctionProto >( fp->duplicate() ) );
+    }
 }
 
-void WCFiletype::setDoubleClickActions(List *ndca)
+void WCFiletype::setShowActions( const command_list_t &nsa )
 {
-  freecoms(doubleclickaction);
-  FunctionProto *fp;
-  int id=ndca->initEnum();
-  fp=(FunctionProto*)ndca->getFirstElement(id);
-  while(fp!=NULL) {
-    doubleclickaction->addElement(fp->duplicate());
-    fp=(FunctionProto*)ndca->getNextElement(id);
-  }
-  ndca->closeEnum(id);
+    showaction.clear();
+    for ( auto &fp : nsa ) {
+        showaction.push_back( std::shared_ptr< FunctionProto >( fp->duplicate() ) );
+    }
 }
 
-void WCFiletype::addComDoubleClickAction( const FunctionProto *fp )
+void WCFiletype::setRawShowActions( const command_list_t &nrsa )
 {
-  if ( fp == NULL ) return;
-  doubleclickaction->addElement( fp->duplicate() );
+    rawshowaction.clear();
+    for ( auto &fp : nrsa ) {
+        rawshowaction.push_back( std::shared_ptr< FunctionProto >( fp->duplicate() ) );
+    }
 }
 
-void WCFiletype::setShowActions(List *nsa)
-{
-  freecoms(showaction);
-  FunctionProto *fp;
-  int id=nsa->initEnum();
-  fp=(FunctionProto*)nsa->getFirstElement(id);
-  while(fp!=NULL) {
-    showaction->addElement(fp->duplicate());
-    fp=(FunctionProto*)nsa->getNextElement(id);
-  }
-  nsa->closeEnum(id);
-}
-
-void WCFiletype::addComShowAction( const FunctionProto *fp )
-{
-  if ( fp == NULL ) return;
-  showaction->addElement( fp->duplicate() );
-}
-
-void WCFiletype::setRawShowActions(List *nrsa)
-{
-  freecoms(rawshowaction);
-  FunctionProto *fp;
-  int id=nrsa->initEnum();
-  fp=(FunctionProto*)nrsa->getFirstElement(id);
-  while(fp!=NULL) {
-    rawshowaction->addElement(fp->duplicate());
-    fp=(FunctionProto*)nrsa->getNextElement(id);
-  }
-  nrsa->closeEnum(id);
-}
-
-void WCFiletype::addComRawShowAction( const FunctionProto *fp )
-{
-  if ( fp == NULL ) return;
-  rawshowaction->addElement( fp->duplicate() );
-}
-
-void WCFiletype::setUserActions(int uid,List *nua)
+void WCFiletype::setUserActions(int uid, const command_list_t &nua )
 {
   if((uid<0)||(uid>9)) return;
-  freecoms(useraction[uid]);
-  FunctionProto *fp;
-  int id=nua->initEnum();
-  fp=(FunctionProto*)nua->getFirstElement(id);
-  while(fp!=NULL) {
-    useraction[uid]->addElement(fp->duplicate());
-    fp=(FunctionProto*)nua->getNextElement(id);
-  }
-  nua->closeEnum(id);
-}
 
-void WCFiletype::addComUserAction( int i, const FunctionProto * fp )
-{
-  if ( fp == NULL ) return;
-  if ( ( i < 0 ) || ( i > 9 ) ) return;
-  useraction[i]->addElement( fp->duplicate() );
+    useraction[uid].clear();
+    for ( auto &fp : nua ) {
+        useraction[uid].push_back( std::shared_ptr< FunctionProto >( fp->duplicate() ) );
+    }
 }
 
 void WCFiletype::setPatternIgnoreCase( bool nv )
@@ -755,90 +673,59 @@ bool WCFiletype::save(Datei *fh) const
 
   fh->configOpenSection( "ftcommands" );
 
-  int id;
-  FunctionProto *f1;
-
-  if ( dndaction->size() > 0 ) {
+  if ( ! dndaction.empty() ) {
       fh->configOpenSection( "dnd" );
-      id = dndaction->initEnum();
-      f1 = (FunctionProto*)dndaction->getFirstElement( id );
-      while ( f1 != NULL ) {
-          FunctionProto::presave( fh, f1 );
-          f1 = (FunctionProto*)dndaction->getNextElement( id );
+      for ( auto &f1 : dndaction ) {
+          FunctionProto::presave( fh, f1.get() );
       }
-      dndaction->closeEnum( id );
       fh->configCloseSection(); //dnd
   }
 
-  if ( doubleclickaction->size() > 0 ) {
+  if ( ! doubleclickaction.empty() ) {
       fh->configOpenSection( "doubleclick" );
-      id = doubleclickaction->initEnum();
-      f1 = (FunctionProto*)doubleclickaction->getFirstElement( id );
-      while ( f1 != NULL ) {
-          FunctionProto::presave( fh, f1 );
-          f1 = (FunctionProto*)doubleclickaction->getNextElement( id );
+      for ( auto &f1 : doubleclickaction ) {
+          FunctionProto::presave( fh, f1.get() );
       }
-      doubleclickaction->closeEnum( id );
       fh->configCloseSection(); //doubleclick
   }
 
-  if ( showaction->size() > 0 ) {
+  if ( ! showaction.empty() ) {
       fh->configOpenSection( "show" );
-      id = showaction->initEnum();
-      f1 = (FunctionProto*)showaction->getFirstElement( id );
-      while ( f1 != NULL ) {
-          FunctionProto::presave( fh, f1 );
-          f1 = (FunctionProto*)showaction->getNextElement( id );
+      for ( auto &f1 : showaction ) {
+          FunctionProto::presave( fh, f1.get() );
       }
-      showaction->closeEnum( id );
       fh->configCloseSection(); //show
   }
   
-  if ( rawshowaction->size() > 0 ) {
+  if ( ! rawshowaction.empty() ) {
       fh->configOpenSection( "rawshow" );
-      id = rawshowaction->initEnum();
-      f1 = (FunctionProto*)rawshowaction->getFirstElement( id );
-      while ( f1 != NULL ) {
-          FunctionProto::presave( fh, f1 );
-          f1 = (FunctionProto*)rawshowaction->getNextElement( id );
+      for ( auto &f1 : rawshowaction ) {
+          FunctionProto::presave( fh, f1.get() );
       }
-      rawshowaction->closeEnum( id );
       fh->configCloseSection(); //rawshow
   }
 
   for ( i = 0; i < 10; i++ ) {
-      if ( useraction[i]->size() > 0 ) {
+      if ( ! useraction[i].empty() ) {
           sprintf( buf, "%d", i );
           str1 = "user ";
           str1 += buf;
           fh->configOpenSection( str1.c_str() );
-          id = useraction[i]->initEnum();
-          f1 = (FunctionProto*)useraction[i]->getFirstElement( id );
-          while ( f1 != NULL ) {
-              FunctionProto::presave( fh, f1 );
-              f1 = (FunctionProto*)useraction[i]->getNextElement( id );
+          for ( auto &f1 : useraction[i] ) {
+              FunctionProto::presave( fh, f1.get() );
           }
-          useraction[i]->closeEnum( id );
           fh->configCloseSection(); //user x
       }
   }
 
-  std::map<std::string, List *>::const_iterator it2;
-
-  for ( it2 = customaction.begin();
-        it2 != customaction.end();
-        it2++ ) {
+  for ( auto &key : customaction ) {
       fh->configOpenSection( "customaction" );
-      fh->configPutPairString( "name", it2->first.c_str() );
+      fh->configPutPairString( "name", key.first.c_str() );
 
       fh->configOpenSection( "commands" );
-      id = it2->second->initEnum();
-      f1 = (FunctionProto*)it2->second->getFirstElement( id );
-      while ( f1 != NULL ) {
-          FunctionProto::presave( fh, f1 );
-          f1 = (FunctionProto*)it2->second->getNextElement( id );
+      for ( auto &f1 : key.second ) {
+          FunctionProto::presave( fh, f1.get() );
       }
-      it2->second->closeEnum( id );
       fh->configCloseSection(); //commands
       fh->configCloseSection(); //customaction
   }
@@ -900,6 +787,10 @@ bool WCFiletype::save(Datei *fh) const
       fh->configPutPairBool( "pattern_is_comma_separated", true );
   }
 
+  if ( m_priority != 0 ) {
+      fh->configPutPairNum( "priority", m_priority );
+  }
+
   return true;
 }
 
@@ -910,8 +801,8 @@ void WCFiletype::fillNamesOfCustomActions( bool recursive, std::list<std::string
     std::map<std::string,List*>::iterator it1;
     
     //TODO iterate through names and check existence
-    for ( it1 = customaction.begin(); it1 != customaction.end(); it1++ ) {
-        names.push_back( it1->first );
+    for ( auto &key : customaction ) {
+        names.push_back( key.first );
     }
     
     if ( recursive == true && parentType != NULL ) {
@@ -919,76 +810,47 @@ void WCFiletype::fillNamesOfCustomActions( bool recursive, std::list<std::string
     }
 }
 
-List *WCFiletype::getActionList( bool recursive, const ActionDescr &descr )
+command_list_t WCFiletype::getActionList( bool recursive, const ActionDescr &descr )
 {
-    List *res_list = NULL;
-    
-    res_list = descr.get_action_list( this );
+    auto res_list = descr.get_action_list( this );
     if ( recursive == true && getParentType() != NULL &&
-         ( res_list == NULL || res_list->size() < 1 ) ) {
+         ( res_list.empty() ) ) {
         res_list = getParentType()->getActionList( recursive, descr );
     }
     return res_list;
 }
 
-List *WCFiletype::getCustomActions( const std::string &customname )
+std::pair< command_list_t, bool > WCFiletype::getCustomActions( const std::string &customname )
 {
     if ( customaction.count( customname ) > 0 ) {
-        return customaction[customname];
+        return std::make_pair( customaction[customname], true );
     }
-    return NULL;
+    return std::make_pair( command_list_t{}, false );
 }
 
-void WCFiletype::setCustomActions( const std::string &customname, List *nua )
+void WCFiletype::setCustomActions( const std::string &customname, const command_list_t &nua )
 {
-    if ( nua == NULL ) return;
-
     if ( customaction.count( customname ) > 0 ) {
-        freecoms( customaction[customname] );
+        customaction[customname].clear();
     } else {
-        customaction[customname] = new List();
+        customaction[customname] = {};
     }
-    FunctionProto *fp;
-    int id = nua->initEnum();
-    fp = (FunctionProto*)nua->getFirstElement( id );
-    while ( fp != NULL ) {
-        customaction[customname]->addElement( fp->duplicate() );
-        fp = (FunctionProto*)nua->getNextElement( id );
-    }
-    nua->closeEnum( id );
-}
 
-void WCFiletype::addComCustomAction( const std::string &customname, const FunctionProto *fp )
-{
-    if ( fp == NULL ) return;
- 
-    if ( customaction.count( customname ) > 0 ) {
-        freecoms( customaction[customname] );
-    } else {
-        customaction[customname] = new List();
+    for ( auto &fp : nua ) {
+        customaction[customname].push_back( std::shared_ptr< FunctionProto >( fp->duplicate() ) );
     }
-    customaction[customname]->addElement( fp->duplicate() );
 }
 
 void WCFiletype::removeCustomAction( const std::string &customname )
 {
     if ( customaction.count( customname ) > 0 ) {
-        freecoms( customaction[customname] );
-	delete customaction[customname];
-	customaction.erase( customname );
+        customaction[customname].clear();
+        customaction.erase( customname );
     }
 }
 
 void WCFiletype::clearCustomActions()
 {
-    std::map<std::string, List *>::iterator it1;
-    
-    for ( it1 = customaction.begin();
-	  it1 != customaction.end();
-	  it1++ ) {
-	freecoms( it1->second );
-	delete it1->second;
-    }
     customaction.clear();
 }
 
@@ -1089,3 +951,13 @@ bool WCFiletype::nameContainsFlags() const
 {
     return m_name_contains_flags;
 }
+
+int WCFiletype::priority() const
+{
+    return m_priority;
+}
+
+void WCFiletype::setPriority( int prio )
+{
+    m_priority = prio;
+}
