@@ -1,6 +1,6 @@
 /* worker.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2020 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -42,6 +42,7 @@
 #include "worker_types.h"
 #include "existence_test.hh"
 #include "wconfig_initial_command.hh"
+#include "file_command_log.hh"
 
 class ListerMode;
 class Lister;
@@ -216,11 +217,15 @@ public:
     bool storePathPers( const std::string &path,
                         const std::string &command_str,
                         time_t ts );
+    bool storePathPersArchive( const std::string &path, time_t ts );
     void removePathPers( const std::string &path );
+    void removePathPersArchive( const std::string &path );
     void removePathProgramPers( const std::string &program );
     DeepPathStore getPathsPers();
+    DeepPathStore getPathsPersArchive();
     std::map< std::string, DeepPathStore > getPathProgramsPers();
     loff_t getSizeOfPersPaths() const;
+    loff_t getSizeOfPersPathsArchive() const;
     loff_t getSizeOfPersPathPrograms() const;
     void relocateEntriesPers( const std::string &dest,
                               const std::string &source,
@@ -260,7 +265,7 @@ public:
     void enqueueFuture( std::future< ExistenceTest::existence_state_t > f );
 
     std::list< WorkerTypes::reg_command_info_t > getListOfCommandsWithoutArgs() const;
-    WorkerTypes::reg_command_info_t getCommandInfo( const std::string &command ) const;
+    WorkerTypes::reg_command_info_t getCommandInfo( const std::string &command, bool args ) const;
 
     static std::string getTerminalCommand( const std::string &cmdfile,
                                            const std::string &outfile,
@@ -268,6 +273,15 @@ public:
 
     void issueConfigOpen( const std::string &panel );
     void issueConfigOpen( const std::string &panel, const WConfigInitialCommand &cmd );
+
+    std::shared_ptr< WPUContext > getCurrentWPU();
+
+    static void pushCommandLog( const std::string &command,
+                                const std::string &path,
+                                const std::string &description );
+
+    std::list< FileCommandLog::Entry > getCommandLog() const;
+    void clearCommandLog();
 protected:
   void createMainWin();
   void setupMainWin();
@@ -469,8 +483,13 @@ protected:
                    const std::list< WCHotkey * > &hotkey_list,
                    const std::list< WCPath * > &path_list );
 
+    std::string getButtonHelpText( int relative_pos_in_bank ) const;
+
+    void setCurrentWPU( std::shared_ptr< WPUContext > wpu );
+
     DeepPathStore m_path_store;
     std::unique_ptr< PersDeepPathStore > m_pers_path_store;
+    std::unique_ptr< PersDeepPathStore > m_pers_path_store_archive;
 
     std::string m_current_hint;
     time_t m_hint_time;
@@ -507,6 +526,11 @@ protected:
 
     TimedText m_help_message;
     bool m_help_active = false;
+
+    std::weak_ptr< WPUContext > m_current_wpu;
+
+    static std::unique_ptr< FileCommandLog > m_command_log;
+    static std::mutex m_command_log_mutex;
 };
 
 #endif
