@@ -1,6 +1,6 @@
 /* wpucontext.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2002-2020 Ralf Hoffmann.
+ * Copyright (C) 2002-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -42,57 +42,45 @@
 
 WPUStack::WPUStack()
 {
-  s = 10;
-  elems = 0;
-  stack = new std::string[s];
 }
 
 WPUStack::~WPUStack()
 {
-  delete [] stack;
 }
 
 void WPUStack::push( const std::string &elem )
 {
-  if ( elems >= s ) raise();
-  stack[elems++] = elem;
+    stack.push_back( elem );
 }
 
 std::string WPUStack::pop()
 {
-  if ( elems < 1 ) return "";
-  return stack[--elems];
+    if ( stack.empty() ) return "";
+    std::string s = stack.back();
+    stack.pop_back();
+    return s;
 }
 
-std::string WPUStack::top()
+std::string WPUStack::top() const
 {
-  if ( elems < 1 ) return "";
-  return stack[elems - 1];
+    if ( stack.empty() ) return "";
+    return stack.back();
 }
 
-bool WPUStack::isEmpty()
+std::string WPUStack::stackEntry( int pos ) const
 {
-  if ( elems < 1 ) return true;
-  return false;
+    if ( pos < 0 || (size_t)pos >= stack.size() ) return "";
+    return stack.at( pos );
 }
 
-int WPUStack::size()
+bool WPUStack::isEmpty() const
 {
-  return elems;
+    return stack.empty();
 }
 
-void WPUStack::raise()
+size_t WPUStack::size() const
 {
-  std::string *newstack;
-  int i, old_s;
-  
-  old_s = s;
-  s *= 2;
-  newstack = new std::string[s];
-  for ( i = 0; i < old_s; i++ )
-    newstack[i] = stack[i];
-  delete [] stack;
-  stack = newstack;
+    return stack.size();
 }
 
 /**************
@@ -120,7 +108,6 @@ WPUContext::WPUContext( Worker *w, const command_list_t &coms, ActionMessage *ms
 {
   struct wpucontext_deftokens *p;
 
-  m_reference_count = 1;
   nr_of_stacks = 0;
   stacks = NULL;
   pc = 0;
@@ -265,6 +252,16 @@ std::string WPUContext::top( int nr )
   return stacks[ nr ]->top();
 }
 
+std::string WPUContext::stackEntry( int nr, int pos )
+{
+    if ( nr >= nr_of_stacks ) {
+        lasterror = WPU_LASTERROR_INVALID_STACK;
+        return "";
+    }
+
+    return stacks[ nr ]->stackEntry( pos );
+}
+
 bool WPUContext::isEmpty( int nr )
 {
   if ( nr >= nr_of_stacks ) buildStack( nr );
@@ -274,10 +271,10 @@ bool WPUContext::isEmpty( int nr )
 int WPUContext::size( int nr )
 {
   if ( nr >= nr_of_stacks ) buildStack( nr );
-  return stacks[ nr ]->size();
+  return (int)stacks[ nr ]->size();
 }
 
-int WPUContext::next( ActionMessage *msg )
+int WPUContext::next( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   AGUIX *aguix = worker->getAGUIX();
   AGMessage *agmsg;
@@ -286,7 +283,7 @@ int WPUContext::next( ActionMessage *msg )
   if ( pc >= acoms.size() ) return 1;
   auto &fp = acoms.at( pc );
   if ( fp ) {
-    fp->run( this, msg );
+    fp->run( wpu, msg );
     pc++;
     
     if ( userwin != NULL ) {
@@ -1318,16 +1315,6 @@ std::string WPUContext::getTempName4File( const char *filename )
   return e.getTempName();
 }
 
-void WPUContext::ref()
-{
-    m_reference_count++;
-}
-
-int WPUContext::unref()
-{
-    return --m_reference_count;
-}
-
 WPUProgressCallback::WPUProgressCallback( Worker *w ) : m_worker( w ), m_complete_sizes( 0 ), m_bytes_already_copied( 0 )
 {
     m_last_update = time( NULL );
@@ -1680,6 +1667,10 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
                         if ( ! res_cli_str.add_str( cwd, force_noquoting ) ) {
                             ende = true;
                         }
+
+                        Worker::pushCommandLog( catalog.getLocale( 1416 ),
+                                                cwd.c_str(),
+                                                cwd.c_str() );
                     }
                 }
             } else if ( strncmp( buf1, "op", 2 ) == 0 ) {
@@ -1691,6 +1682,10 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
                         if ( ! res_cli_str.add_str( cwd, force_noquoting ) ) {
                             ende = true;
                         }
+
+                        Worker::pushCommandLog( catalog.getLocale( 1416 ),
+                                                cwd.c_str(),
+                                                cwd.c_str() );
                     }
                 }
             } else if ( strncmp( buf1, "lp", 2 ) == 0 ) {
@@ -1707,6 +1702,10 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
                         if ( ! res_cli_str.add_str( cwd, force_noquoting ) ) {
                             ende = true;
                         }
+
+                        Worker::pushCommandLog( catalog.getLocale( 1416 ),
+                                                cwd.c_str(),
+                                                cwd.c_str() );
                     }
                 }
             } else if ( strncmp( buf1, "rp", 2 ) == 0 ) {
@@ -1723,6 +1722,10 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
                         if ( ! res_cli_str.add_str( cwd, force_noquoting ) ) {
                             ende = true;
                         }
+
+                        Worker::pushCommandLog( catalog.getLocale( 1416 ),
+                                                cwd.c_str(),
+                                                cwd.c_str() );
                     }
                 }
             } else if ( strncmp( buf1, "dp", 2 ) == 0 ) {
@@ -1734,6 +1737,10 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
                         if ( ! res_cli_str.add_str( cwd, force_noquoting ) ) {
                             ende = true;
                         }
+
+                        Worker::pushCommandLog( catalog.getLocale( 1416 ),
+                                                cwd.c_str(),
+                                                cwd.c_str() );
                     }
                 }
             } else if ( strncmp( buf1, "v", 1 ) == 0 ) {
@@ -1870,6 +1877,10 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
                                 if ( ende == false && ! res_cli_str.add_str( name_cstring, force_noquoting ) ) {
                                     ende = true;
                                 }
+
+                                Worker::pushCommandLog( catalog.getLocale( 1416 ),
+                                                        efe1->getFullname( false ),
+                                                        efe1->getFullname( false ) );
                             }               
                             if ( ende == true ) break;
                         }
